package kutils

import (
	"context"
	"crypto/tls"
	"crypto/x509"
	"errors"
	"github.com/Rican7/retry"
	"github.com/Rican7/retry/strategy"
	"github.com/Shopify/sarama"
	log "github.com/sirupsen/logrus"
	"io/ioutil"
	"strings"
	"time"
)

type userSettings struct {
	user		string
	password	string
	groupId		string
	certPath	string
}

type ConsumerContext struct {
	KafkaCtx
	userSettings
}

//Connection to kafka function
func connectReader(consumerCtx ConsumerContext) (*sarama.ConsumerGroup, error) {
	conf := sarama.NewConfig()
	conf.Consumer.Return.Errors = true
	conf.Version = sarama.MaxVersion
	conf.ClientID = "sasl_scram_client"
	conf.Metadata.Full = true
	conf.Net.SASL.Enable = true
	conf.Net.SASL.User = consumerCtx.user
	conf.Net.SASL.Password = consumerCtx.password
	conf.Net.SASL.Handshake = true
	conf.Net.SASL.SCRAMClientGeneratorFunc = func() sarama.SCRAMClient { return &XDGSCRAMClient{HashGeneratorFcn: SHA512} }
	conf.Net.SASL.Mechanism = sarama.SASLMechanism(sarama.SASLTypeSCRAMSHA512)
	conf.Consumer.Group.Rebalance.Strategy = sarama.BalanceStrategyRange
	conf.Consumer.Offsets.Initial = sarama.OffsetOldest

	certs := x509.NewCertPool()
	pemPath := consumerCtx.certPath
	pemData, err := ioutil.ReadFile(pemPath)
	if err != nil {
		//consumerCtx.Error = err
		return nil, err
	}
	certs.AppendCertsFromPEM(pemData)

	conf.Net.TLS.Enable = true
	conf.Net.TLS.Config = &tls.Config{
		InsecureSkipVerify: true,
		RootCAs: certs,
	}

	client, err := sarama.NewConsumerGroup(strings.Split(consumerCtx.connection.host, ","), consumerCtx.groupId, conf)
	if err != nil {
		//consumerCtx.Error = err
		return nil, err
	}
	return &client, nil
}

//Функции чтения

func (ctx *KafkaCtx) SetReader(config sarama.ConsumerGroup) *ConsumerContext {
	consumerCtx := ConsumerContext{KafkaCtx:*ctx}
	consumerCtx.r = config
	return &consumerCtx
}

func (ctx *KafkaCtx) ReaderConf(user, password, groupId, certificatePath string) *ConsumerContext {
	consumerCtx := ConsumerContext{KafkaCtx:*ctx}
	if consumerCtx.Error != nil {
		return &consumerCtx
	}
	consumerCtx.user = user
	consumerCtx.password = password
	consumerCtx.groupId = groupId
	consumerCtx.certPath = certificatePath

	//consumerCtx.r = client
	return &consumerCtx
}

func (ctx *ConsumerContext) StartConsume(settingFunc func(sarama.ConsumerGroupClaim) error, worker func(message *sarama.ConsumerMessage) KutilsErrorI) *ConsumerContext {
	if ctx.Error != nil {
		return ctx
	}
	consumer := Consumer{
		ready: make(chan bool),
		f: settingFunc,
		worker: worker,
	}

	go func() {
		for {
			err := retry.Retry(func(attempt uint) error {
				if ctx == nil {
					return errors.New("null context exception")
				}
				var err error = nil
				for err == nil {
					client, err := connectReader(*ctx)
					if err != nil {
						return err
					}
					ctx.r = *client
					err = ctx.r.Consume(context.Background(), []string{ctx.connection.topic}, &consumer)
					if err != nil {
						return err
					}
					//пока ошибка = nil продолжаем работу
					consumer.ready = make(chan bool)
				}
				// иначе восстанавливаемся
				return err
			}, strategy.Limit(5), strategy.Wait(time.Minute))
			if err != nil {
				log.Fatal(err)
			}
		}
	}()
	<-consumer.ready
	return ctx
}

func (ctx *ConsumerContext) CloseConsumer() *ConsumerContext {
	if ctx.Error != nil {
		return ctx
	}
	ctx.Error = ctx.r.Close()
	return ctx
}